import struct, fcntl, termios, subprocess

from polycal5 import KEY_LEFT, KEY_UP, KEY_DOWN, KEY_RIGHT, KEY_ENTER, \
                      KEY_ESC, KEY_4, KEY_5, KEY_6

_x_offset = 0
_keys = {
  "w": KEY_UP,
  "a": KEY_LEFT,
  "s": KEY_DOWN,
  "d": KEY_RIGHT,
  "q": KEY_4,
  "e": KEY_6,
  "1": KEY_4,
  "2": KEY_DOWN,
  "3": KEY_6,
  "4": KEY_LEFT,
  "5": KEY_5,
  "6": KEY_RIGHT,
  "7": KEY_4,
  "8": KEY_UP,
  "9": KEY_6,
  "\n": KEY_ENTER,
  "\x1b": KEY_ESC,
  "x": KEY_ESC,
  "\x18": KEY_ESC
}

def polycalc_pc_init(width=0, height=0, scale=1):
  global HEIGHT, WIDTH, _x_offset
  height, width = struct.unpack('hh', fcntl.ioctl(0, termios.TIOCGWINSZ, '1234'))
  max_width = 2 * height

  HEIGHT = height - 1
  WIDTH = max_width if width > max_width else width
  _x_offset = (width - WIDTH) // 2 + 1

  reset_term()
  hide_cursor()
  poly_clean_screen()

def reset_term():
    subprocess.run(["stty", "sane"])
    print("\x1b[?25h\x1b[0m\x1b[2J", end="")

def hide_cursor():
    print("\x1b[?25l", end="")

def set_color(color):
  r, g, b = color
  print(f"\x1b[0;48;2;{r};{g};{b}m", end="")

def set_location(x, y):
  print(f"\x1b[{y+1};{_x_offset + x}H", end="")

def poly_set_pixel(x, y, color):
  if x < 0 or x >= WIDTH or y < 0 or y >= HEIGHT:
    return
  _screen[WIDTH * y + x] = color

def poly_fill_rect(x, y, w, h, color):
  for i in range(h):
    off = WIDTH * (y + i) + x
    for j in range(w):
      _screen[off + j] = color

def poly_clean_screen():
  global _screen
  _screen = [(0, 0, 0)] * (WIDTH * HEIGHT)

def poly_show():
  set_location(0, 0)
  y = 0
  x = 0
  for color in _screen:
    set_color(color)
    print(" ", end="")
    x += 1
    if x >= WIDTH:
      x = 0
      y += 1
      set_location(x, y)
  print("\x1b[0m", end="")

def poly_pause():
  raise Exception("not implemented")

def poly_wait_key():
  hide_cursor()
  while True:
    c = subprocess.run(["bash", "-c", "stty cbreak -echo >/dev/null;read -n1 KEY;echo $KEY;stty sane"], stdout=subprocess.PIPE).stdout.decode()[0]
    try:
      key = _keys[c]
      if key == KEY_ESC:
        reset_term()
      return key
    except:
      pass

def poly_get_key():
  raise Exception("not implemented")

def poly_test_key(_):
  raise Exception("not implemented")

def poly_ignore_events():
  pass

def poly_esc_key():
  return None

def poly_wait_release():
  raise Exception("not implemented")
